from operator import add

class Dot (object):
	"""Minimal 2d math."""
	def __init__(self, x, y):
		self.x = float(x)
		self.y = float(y)
	
	def __mul__(self, k):
		return Dot(self.x * k, self.y * k)

	def __add__(self, d):
		return Dot(self.x + d.x, self.y + d.y)

	def tupi(self):
		return (int(self.x), int(self.y))

	def __repr__(self):
		return '(%.4f %.4f)' % (self.x, self.y)


class Curve (object):
	"""Bezier curve with four control points.

	This can be easily extended to support more control points.
	"""

	def __init__(self, a, b, c, d):
		self.points = map(self.convert, [a, b, c, d])

		#                  n-1
		# (i, n - i - 1, (     ))
		#                   i
		# where i is in range(0, n)
		# and n is the number of control points.
		self.params = [(0, 3, 1), (1, 2, 3), (2, 1, 3), (3, 0, 1)]
	
	def convert(self, p):
		return Dot(*p)

	def interpolate(self, t):
		s = 1 - t
		return reduce(add, [self.points[i] * k * t**i * s**j for i, j, k in self.params])


if __name__ == '__main__':
	c = Curve((0, 0), (1, 0), (0, 1), (1, 1))
	for i in range(11):
		t = i / 10.
		print c.interpolate(t)
